#Sun and Moon almanac

import math

#Trigonometric functions including haversine (angles measured in degrees)
def sind(x):
    return math.sin(math.radians(x))
    
def cosd(x):
    return math.cos(math.radians(x))
        
def tand(x):
    return math.tan(math.radians(x))

def havd(x):
    a=math.sin(math.radians(x/2))
    return a*a

#Inverse trigonometric functions including inverse haversine (results measured in degrees)
def asind(x):
    return math.degrees(math.asin(x))

def acosd(x):
    return math.degrees(math.acos(x))

def atand(x):
    return math.degrees(math.atan(x))

def atand2(x,y):
    return math.degrees(math.atan2(x,y))

def ahavd(x):
    return math.degrees(2*math.asin(math.sqrt(x)))

#Date to Julian Day
def date2jd(y,m,d):       
    if m==1 or m==2:
        y-=1
        m+=12
    a=int(y/100)
    b=2-a+int(a/4)
    J=int(365.25*(y+4716))+int(30.6001*(m+1))+d+b-1524.5
    return J

#Julian Day to Date
def jd2date(JD):
    z=int(JD+0.5)
    f=JD+0.5-z
    alpha=int((z-1867216.25)/36524.25)
    a=z+1+alpha-int(alpha/4)
    b=a+1524
    c=int((b-122.1)/365.25)
    d=int(365.25*c)
    e=int((b-d)/30.6001)
    day=b-d-int(30.6001*e)+f
    if e<14:
        month=e-1
    elif e==14 or e==15:
        month=e-13
    if month>2:
        year=c-4716
    elif month==1 or month==2:
        year=c-4715
    return day,month,year

#Decimal to ddd.mm.ss.s or hh.mm.ss.s
def dd2ddmmss_s(x):
    u=int(x)
    v=int(60*(x-u))
    w=round(3600*(x-u-v/60),1)
    if w==60.0:
        w=0
        v+=1
    if v==60:
        v=0
        u+=1
    u=str(u)
    v=str(v)
    w=str(w)
    if len(u)==1:
        u="0"+u
    if len(v)==1:
        v="0"+v
    if len(w)==1 or len(w)==3:
        w="0"+w 
    return(u,v,w)
        
#Decimal to ddd.mm.ss or hh.mm.ss
def dd2dms(x):
    u=int(x)
    v=int(60*(x-u))
    w=round(3600*(x-u-v/60))
    if w==60:
        w=0
        v+=1
    if v==60:
        v=0
        u+=1
    u=str(u)
    v=str(v)
    w=str(w)
    if len(u)==1:
        u="0"+u
    if len(v)==1:
        v="0"+v
    if len(w)==1:
        w="0"+w 
    return(u,v,w)

#Decimal to hh.mm
def dh2hm(x):
    u=int(x)
    v=round(60*(x-u))
    if v==60:
        v=0
        u+=1
    u=str(u)
    if len(u)==1:
        u="0"+u
    v=str(v)
    if len(v)==1:
        v="0"+v
    return(u,v)

#Calculates day of week
def weekday(x):
    wd=""
    y=int(x)
    if y%7==0:
        wd="TUESDAY"
    elif y%7==1:
        wd="WEDNESDAY"
    elif y%7==2:
        wd="THURSDAY"
    elif y%7==3:
        wd="FRIDAY"
    elif y%7==4:
        wd="SATURDAY"
    elif y%7==5:
        wd="SUNDAY"
    elif y%7==6:
        wd="MONDAY"
    return wd

#Calculates name of month
def monthname(x):
    mn=""
    if x==1:
        mn="JANUARY"
    elif x==2:
        mn="FEBRUARY"
    elif x==3:
        mn="MARCH"
    elif x==4:
        mn="APRIL"
    elif x==5:
        mn="MAY"
    elif x==6:
        mn="JUNE"
    elif x==7:
        mn="JULY"
    elif x==8:
        mn="AUGUST"
    elif x==9:
        mn="SEPTEMBER"
    elif x==10:
        mn="OCTOBER"
    elif x==11:
        mn="NOVEMBER"
    elif x==12:
        mn="DECEMBER"
    return mn

#Calculates Julian date, Julian Century, etc.
def time(J,UT,Delta_T):
    JD=J+UT/24
    JDE=JD+Delta_T/86400   
    T=(JD-2451545)/36525
    TE=(JDE-2451545)/36525
    Tau=TE/10
    return JD,JDE,T,TE,Tau

#Normalizes date and UT
def normtime(J,UT):
    while UT<0:
        J-=1
        UT+=24
    while UT>24:
        J+=1
        UT-=24
    return J,UT

#Normalizes meridian angle
def normt(t):
    if t>180:
        t-=360
    elif t<=-180:
        t+=360
    return t

#Calculates change of altitude of the Moon in degrees per hour
def moonslope(J,UT,Delta_T):
    UTlow=UT-0.005
    RA1,Dec1,GHA1,HP1,SD1,k1,LMoon1=mooncoordinates(J,UTlow,Delta_T)
    H1=90-ahavd(havd(Lat-Dec1)+cosd(Lat)*cosd(Dec1)*havd(GHA1+Lon))
    UThi=UT+0.005
    RA2,Dec2,GHA2,HP2,SD2,k2,LMoon2=mooncoordinates(J,UThi,Delta_T)
    H2=90-ahavd(havd(Lat-Dec2)+cosd(Lat)*cosd(Dec2)*havd(GHA2+Lon))
    slope=(H2-H1)*100
    return slope
    
#Nutation in longitude ans obliquity, obliquity of the ecliptic
def nutation(J,UT,Delta_T):
        
    JD,JDE,T,TE,Tau=time(J,UT,Delta_T)
    
    #D=297.85036+445267.11148*TE-0.0019142*TE2+TE3/189474
    D=297.85036+TE*(445267.11148+TE*(TE/189474-0.0019142))
    D%=360
    
    #M=357.52772+35999.05034*TE-0.0001603*TE2-TE3/300000
    M=357.52772+TE*(35999.05034-TE*(0.0001603+TE/300000))
    M%=360
    
    #Mprime=134.96298+477198.867398*TE+0.0086972*TE2+TE3/56250
    Mprime=134.96298+TE*(477198.867398+TE*(0.0086972+TE/56250))
    Mprime%=360
    
    #F=93.27191+483202.017538*TE-0.0036825*TE2+TE3/327270
    F=93.27191+TE*(483202.017538+TE*(TE/327270-0.0036825))
    F%=360
    
    #Omega=125.04452-1934.136261*TE+0.0020708*TE2+TE3/450000
    Omega=125.04452-TE*(1934.136261+TE*(0.0020708+TE/450000))
    Omega%=360
    
    #Coefficients
    cfD=[0,-2,0,0,0,0,-2,0,0,-2,-2,-2,0,2,0,2,0,0,-2,0,2,0,0,-2,0,-2,0,0,2,-2,0,-2,0,0,2,2,0,-2,0,2,2,-2,-2,2,2,0,-2,-2,0,-2,-2,0,-1,-2,1,0,0,-1,0,0,2,0,2]
    cfM=[0,0,0,0,1,0,1,0,0,-1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,2,1,0,-1,0,0,0,1,1,-1,0,0,0,0,0,0,-1,-1,0,0,0,1,0,0,1,0,0,0,-1,1,-1,-1,0,-1]
    cfMprime=[0,0,0,0,0,1,0,0,1,0,1,0,-1,0,1,-1,-1,1,2,-2,0,2,2,1,0,0,-1,0,-1,0,0,1,0,2,-1,1,0,1,0,0,1,2,1,-2,0,1,0,0,2,2,0,1,1,0,0,1,-2,1,1,1,-1,3,0]
    cfF=[0,2,2,0,0,0,2,2,2,2,0,2,2,0,0,2,0,2,0,2,2,2,0,2,2,2,2,0,0,2,0,0,0,-2,2,2,2,0,2,2,0,2,2,0,0,0,2,0,2,0,2,-2,0,0,0,2,2,0,0,2,2,2,2]
    cfOmega=[1,2,2,2,0,0,2,1,2,2,0,1,2,0,1,2,1,1,0,1,2,2,0,2,0,0,1,0,1,2,1,1,1,0,1,2,2,0,2,1,0,2,1,1,1,0,1,1,1,1,1,0,0,0,0,0,2,0,0,2,2,2,2]
    
    sin=[]
    z=0.0
    for i in range(0,63):
        z=sind(cfD[i]*D+cfM[i]*M+cfMprime[i]*Mprime+cfF[i]*F+cfOmega[i]*Omega)
        sin.append(z)
    
    cos=[]
    z=0.0
    for i in range(0,63):
        z=cosd(cfD[i]*D+cfM[i]*M+cfMprime[i]*Mprime+cfF[i]*F+cfOmega[i]*Omega)
        cos.append(z)
    
    cfDpsi=[-171996,-13187,-2274,2062,1426,712,-517,-386,-301,217,-158,129,123,63,63,-59,-58,-51,48,46,-38,-31,29,29,26,-22,21,17,16,-16,-15,-13,-12,11,-10,-8,7,-7,-7,-7,6,6,6,-6,-6,5,-5,-5,-5,4,4,4,-4,-4,-4,3,-3,-3,-3,-3,-3,-3,-3]
    T1=[-174.2,-1.6,-0.2,0.2,-3.4,0.1,1.2,-0.4,0,-0.5,0,0.1,0,0,0.1,0,-0.1,0,0,0,0,0,0,0,0,0,0,-0.1,0,0.1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0]
    cfDeps=[92025,5736,977,-895,54,-7,224,200,129,-95,0,-70,-53,0,-33,26,32,27,0,-24,16,13,0,-12,0,0,-10,0,-8,7,9,7,6,0,5,3,-3,0,3,3,0,-3,-3,3,3,0,3,3,3,0,0,0,0,0,0,0,0,0,0,0,0,0,0]
    T2=[8.9,-3.1,-0.5,0.5,-0.1,0,-0.6,0,-0.1,0.3,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0]
    
    Dpsi=[]
    z=0.0
    for i in range(0,63):
        z=sin[i]*(cfDpsi[i]+TE*T1[i])
        Dpsi.append(z)
        
    DeltaPsi=math.fsum(Dpsi)/36000000
    
    Deps=[]
    z=0.0
    for i in range(0,63):
        z=cos[i]*(cfDeps[i]+TE*T2[i])
        Deps.append(z)
      
    DeltaEps=math.fsum(Deps)/36000000
        
    #Eps0=23.4392911111111+(-46.815*TE-0.00059*TE2+0.001813*TE3)/3600
    Eps0=23.4392911111111+TE*(TE*(TE*0.001813-0.00059)-46.815)/3600
    Eps=Eps0+DeltaEps
    
    return DeltaPsi,DeltaEps,Eps0,Eps

#Calculates GMST, GAST, RA, GHA, Dec, SD, HP, EoT, and ecliptic longitude
def suncoordinates(J,UT,Delta_T):
    
    JD,JDE,T,TE,Tau=time(J,UT,Delta_T)
          
    #L0
    A0=[175347046,3341656,34894,3497,3418,3136,2676,2343,1324,1273,1199,990,902,857,780,753,505,492,357,317,284,271,243,206,205,202,156,132,126,115,103,102,102,99,98,86,85,85,80,79,75,74,74,70,62,61,57,56,56,52,52,51,49,41,41,39,37,37,36,36,33,30,30,25]
    B0=[0,4.6692568,4.6261,2.7441,2.8289,3.6277,4.4181,6.1352,0.7425,2.0371,1.1096,5.233,2.045,3.508,1.179,2.533,4.583,4.205,2.92,5.849,1.899,0.315,0.345,4.806,1.869,2.458,0.833,3.411,1.083,0.645,0.636,0.976,4.267,6.21,0.68,5.98,1.3,3.67,1.81,3.04,1.76,3.5,4.68,0.83,3.98,1.82,2.78,4.39,3.47,0.19,1.33,0.28,0.49,5.37,2.4,6.17,6.04,2.57,1.71,1.78,0.59,0.44,2.74,3.16]
    C0=[0,6283.07585,12566.1517,5753.3849,3.5231,77713.7715,7860.4194,3930.2097,11506.7698,529.691,1577.3435,5884.927,26.298,398.149,5223.694,5507.553,18849.228,775.523,0.067,11790.629,796.298,10977.079,5486.778,2544.314,5573.143,6069.777,213.299,2942.463,20.775,0.98,4694.003,15720.839,7.114,2146.17,155.42,161000.69,6275.96,71430.7,17260.15,12036.46,5088.63,3154.69,801.82,9437.76,8827.39,7084.9,6286.6,14143.5,6279.55,12139.55,1748.02,5856.48,1194.45,8429.24,19651.05,10447.39,10213.29,1059.38,2352.87,6812.77,17789.85,83996.85,1349.87,4690.48]    
         
    sum=[]
    z=0.0
    for i in range(0,64):
        z=A0[i]*math.cos(B0[i]+Tau*C0[i])
        sum.append(z)
    sum_L0=math.fsum(sum)        
    
    #L1
    A1=[628331966747,206059,4303,425,119,109,93,72,68,67,59,56,45,36,29,21,19,19,17,16,16,15,12,12,12,12,11,10,10,9,9,8,6,6]
    B1=[0,2.678235,2.6351,1.59,5.796,2.966,2.59,1.14,1.87,4.41,2.89,2.17,0.4,0.47,2.65,5.34,1.85,4.97,2.99,0.03,1.43,1.21,2.83,3.26,5.27,2.08,0.77,1.3,4.24,2.7,5.64,5.3,2.65,4.67]
    C1=[0,6283.07585,12566.1517,3.523,26.298,1577.344,18849.23,529.69,398.15,5507.55,5223.69,155.42,796.3,775.52,7.11,0.98,5486.78,213.3,6275.96,2544.31,2146.17,10977.08,1748.02,5088.63,1194.45,4694,553.57,6286.6,1349.87,242.73,951.72,2352.87,9437.76,4690.48]
    
    sum=[]
    z=0.0
    for i in range(0,34):
        z=A1[i]*math.cos(B1[i]+Tau*C1[i])
        sum.append(z)
    sum_L1=math.fsum(sum)  
        
    #L2
    A2=[52919,8720,309,27,16,16,10,9,7,5,4,4,3,3,3,3,3,3,2,2]
    B2=[0,1.0721,0.867,0.05,5.19,3.68,0.76,2.06,0.83,4.66,1.03,3.44,5.14,6.05,1.19,6.12,0.31,2.28,4.38,3.75]
    C2=[0,6283.0758,12566.152,3.52,26.3,155.42,18849.23,77713.77,775.52,1577.34,7.11,5573.14,796.3,5507.55,242.73,529.69,398.15,553.57,5223.69,0.98]
    
    sum=[]
    z=0.0
    for i in range(0,20):
        z=A2[i]*math.cos(B2[i]+Tau*C2[i])
        sum.append(z)
    sum_L2=math.fsum(sum)
        
    #L3
    A3=[289,35,17,3,1,1,1]
    B3=[5.844,0,5.49,5.2,4.72,5.3,5.97]
    C3=[6283.076,0,12566.15,155.42,3.52,18849.23,242.73]
    
    sum=[]
    z=0.0
    for i in range(0,7):
        z=A3[i]*math.cos(B3[i]+Tau*C3[i])
        sum.append(z)
    sum_L3=math.fsum(sum)
        
    #L4
    A4=[114,8,1]
    B4=[3.142,4.13,3.84]
    C4=[0,6283.08,12566.15]
    
    sum=[]
    z=0.0
    for i in range(0,3):
        z=A4[i]*math.cos(B4[i]+Tau*C4[i])
        sum.append(z)
    sum_L4=math.fsum(sum)
    
    #L5
    sum_L5=math.cos(3.14)    
    
    #L=(sum_L0+sum_L1*Tau+sum_L2*Tau2+sum_L3*Tau3+sum_L4*Tau4+sum_L5*Tau5)/100000000
    L=(sum_L0+Tau*(sum_L1+Tau*(sum_L2+Tau*(sum_L3+Tau*(sum_L4+Tau*sum_L5)))))/100000000
    
    #B0   
    AA0=[280,102,80,44,32]
    BB0=[3.199,5.422,3.88,3.7,4]
    CC0=[84334.662,5507.553,5223.69,2352.87,1577.34]
    
    sum=[]
    z=0.0
    for i in range(0,5):
        z=AA0[i]*math.cos(BB0[i]+Tau*CC0[i])
        sum.append(z)
    sum_B0=math.fsum(sum)
        
    #B1
    AA1=[9,6]
    BB1=[3.9,1.73]
    CC1=[5507.55,5223.69]
    
    sum=[]
    z=0.0
    for i in range(0,2):
        z=AA1[i]*math.cos(BB1[i]+Tau*CC1[i])
        sum.append(z)
    sum_B1=math.fsum(sum)
        
    #B
    B=(sum_B0+sum_B1*Tau)/100000000
    
    #R0
    AAA0=[100013989,1670700,13956,3084,1628,1576,925,542,472,346,329,307,243,212,186,175,110,98,86,86,65,63,57,56,49,47,45,43,39,38,37,37,36,35,33,32,32,28,28,26]
    BBB0=[0,3.0984635,3.05525,5.1985,1.1739,2.8469,5.453,4.564,3.661,0.964,5.9,0.299,4.273,5.847,5.022,3.012,5.055,0.89,5.69,1.27,0.27,0.92,2.01,5.24,3.25,2.58,5.54,6.01,5.36,2.39,0.83,4.9,1.67,1.84,0.24,0.18,1.78,1.21,1.9,4.59]
    CCC0=[0,6283.07585,12566.1517,77713.7715,5753.3849,7860.4194,11506.77,3930.21,5884.927,5507.553,5223.694,5573.143,11790.629,1577.344,10977.079,18849.228,5486.778,6069.78,15720.84,161000.69,17260.15,529.69,83996.85,71430.7,2544.31,775.52,9437.76,6275.96,4694,8827.39,19651.05,12139.55,12036.46,2942.46,7084.9,5088.63,398.15,6286.6,6279.55,10447.39]
    
    sum=[]
    z=0.0
    for i in range(0,40):
        z=AAA0[i]*math.cos(BBB0[i]+Tau*CCC0[i])
        sum.append(z)
    sum_R0=math.fsum(sum)
        
    
    AAA1=[103019,1721,702,32,31,25,18,10,9,9]
    BBB1=[1.10749,1.0644,3.142,1.02,2.84,1.32,1.42,5.91,1.42,0.27]
    CCC1=[6283.07585,12566.1517,0,18849.23,5507.55,5223.69,1577.34,10977.08,6275.96,5486.78]
    
    sum=[]
    z=0.0
    for i in range(0,10):
        z=AAA1[i]*math.cos(BBB1[i]+Tau*CCC1[i])
        sum.append(z)
    sum_R1=math.fsum(sum)
        
    
    AAA2=[4359,124,12,9,6,3]
    BBB2=[5.7846,5.579,3.14,3.63,1.87,5.47]
    CCC2=[6283.0758,12566.152,0,77713.77,5573.14,18849.23]
    
    sum=[]
    z=0.0
    for i in range(0,6):
        z=AAA2[i]*math.cos(BBB2[i]+Tau*CCC2[i])
        sum.append(z)
    sum_R2=math.fsum(sum)
        
    AAA3=[145,7]
    BBB3=[4.273,3.92]
    CCC3=[6283.076,12566.15]
    
    sum=[]
    z=0.0
    for i in range(0,2):
        z=AAA3[i]*math.cos(BBB3[i]+Tau*CCC3[i])
        sum.append(z)
    sum_R3=math.fsum(sum)
        
    sum_R4=4*math.cos(2.56+Tau*6283.08)    
    
    #R=(sum_R0+sum_R1*Tau+sum_R2*Tau2+sum_R3*Tau3+sum_R4*Tau4)/100000000
    R=(sum_R0+Tau*(sum_R1+Tau*(sum_R2+Tau*(sum_R3+Tau*sum_R4))))/100000000
    
    Ldd=math.degrees(L)%360
    
    L_sun=(Ldd+180)%360
       
    Beta_sun=-math.degrees(B)
    
    #L_prime=(L_sun-1.397*TE-0.00031*TE2)%360
    L_prime=L_sun-TE*(1.397+0.00031*TE)
    L_prime%=360
    
    L_prime_rad=math.radians(L_prime)
    
    Delta_L=(-0.09033+0.03916*(math.cos(L_prime_rad)+math.sin(L_prime_rad))*math.tan(B))/3600
    
    Delta_B=0.03916*(math.cos(L_prime_rad)-math.sin(L_prime_rad))/3600
    
    L_corr=L_sun+Delta_L
    
    B_corr=Beta_sun+Delta_B    
    
    DeltaPsi,DeltaEps,Eps0,Eps=nutation(J,UT,Delta_T)
    
    Abberation=-20.4898/R/3600
    
    Lambd=L_corr+DeltaPsi+Abberation
    LSun=Lambd
    
    RA=atand2(sind(Lambd)*cosd(Eps)-tand(B_corr)*sind(Eps),cosd(Lambd))        
       
    Dec=asind(sind(B_corr)*cosd(Eps)+cosd(B_corr)*sind(Eps)*sind(Lambd))    
      
    #GMST=280.46061837+360.98564736629*(JD-2451545)+0.000387933*T*T-T*T*T/38710000     
    GMST=280.46061837+360.98564736629*(JD-2451545)+T*T*(0.000387933-T/38710000)    
       
    GAST=GMST+DeltaPsi*cosd(Eps)           
    
    GHA=GAST-RA
    
    GHA%=360    
    RA%=360
    GMST%=360
    GAST%=360
    
    #EoT in degrees
    EoT=GHA+180-15*UT
    if EoT>=5:
        EoT-=360
    elif EoT<-5:
        EoT+=360    
       
    SD=959.63/R/60
        
    HP=8.794/R/60  
            
    return GMST,GAST,RA,GHA,Dec,SD,HP,EoT,R,LSun

#Calculates RA, GHA, Dec, SD, HP, k, and ecliptic longitude
def mooncoordinates(J,UT,Delta_T):
    #Time
    JD,JDE,T,TE,Tau=time(J,UT,Delta_T)

    #Mean longitude of the Moon
    Ldash=218.316447+TE*(TE*(TE*(1.85584e-6-TE/65194000)-0.0015786)+481267.88123421)

    #Mean elongation of the Moon
    D=297.8501921+TE*(TE*(TE*(1.83194e-6-TE/113065000)-0.0018819)+445267.1114034)

    #Mean anomaly of the Sun
    M=357.5291092+TE*(TE*(TE/24490000-0.0001536)+35999.0502909)

    #Mean anomaly of the Moon
    Mdash=134.9633964+TE*(TE*(TE*(1.434741e-5-TE/14712000)+0.0087414)+477198.8675055)

    #Moon's argument of latitude
    F=93.272095+TE*(TE*(TE*(TE/8633100000-2.8361e-7)-0.0036539)+483202.0175233)

    #Further arguments (in degrees)
    A_1=119.75+131.849*TE
    A_1%=360
    A_2=53.09+479264.29*TE
    A_2%=360
    A_3=313.45+481266.484*TE
    A_3%=360
    #E=1-0.002516*TE-0.0000074*TE*TE
    E=1-TE*(TE*0.0000074+0.002516)
    
    #Periodic Terms
    #Coefficients for sumL and sumR
    cfD=[0,2,2,0,0,0,2,2,2,2,0,1,0,2,0,0,4,0,4,2,2,1,1,2,2,4,2,0,2,2,1,2,0,0,2,2,2,4,0,3,2,4,0,2,2,2,4,0,4,1,2,0,1,3,4,2,0,1,2,2]
    cfM=[0,0,0,0,1,0,0,-1,0,-1,1,0,1,0,0,0,0,0,0,1,1,0,1,-1,0,0,0,1,0,-1,0,-2,1,2,-2,0,0,-1,0,0,1,-1,2,2,1,-1,0,0,-1,0,1,0,1,0,0,-1,2,1,0,0]
    cfMdash=[1,-1,0,2,0,0,-2,-1,1,0,-1,0,1,0,1,1,-1,3,-2,-1,0,-1,0,1,2,0,-3,-2,-1,-2,1,0,2,0,-1,1,0,-1,2,-1,1,-2,-1,-1,-2,0,1,4,0,-2,0,2,1,-2,-3,2,1,-1,3,-1]
    cfF=[0,0,0,0,0,2,0,0,0,0,0,0,0,-2,2,-2,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,0,0,0,-2,2,0,2,0,0,0,0,0,0,-2,0,0,0,0,-2,-2,0,0,0,0,0,0,0,-2]
    
    cfSine=[6288774,1274027,658314,213618,-185116,-114332,58793,57066,53322,45758,-40923,-34720,-30383,15327,-12528,10980,10675,10034,8548,-7888,-6766,-5163,4987,4036,3994,3861,3665,-2689,-2602,2390,-2348,2236,-2120,-2069,2048,-1773,-1595,1215,-1110,-892,-810,759,-713,-700,691,596,549,537,520,-487,-399,-381,351,-340,330,327,-323,299,294,0]
    cfCosine=[-20905355,-3699111,-2955968,-569925,48888,-3149,246158,-152138,-170733,-204586,-129620,108743,104755,10321,0,79661,-34782,-23210,-21636,24208,30824,-8379,-16675,-12831,-10445,-11650,14403,-7003,0,10056,6322,-9884,5751,0,-4950,4130,0,-3958,0,3258,2616,-1897,-2117,2354,0,0,-1423,-1117,-1571,-1739,0,-4421,0,0,0,0,1165,0,0,8752]
    
    #sumL
    sine=[]
    z=0.0
    for i in range(0,60):
        z=cfSine[i]*sind(cfD[i]*D+cfM[i]*M+cfMdash[i]*Mdash+cfF[i]*F)
        if cfM[i]==1 or cfM[i]==-1:
            z*=E
        elif cfM[i]==2 or cfM[i]==-2:
            z=z*E*E
            
        sine.append(z)

    sumL=math.fsum(sine)
    
    sumL=sumL+3958*sind(A_1)+1962*sind(Ldash-F)+318*sind(A_2)
    
    lambdaMoon=Ldash+sumL/1000000
    lambdaMoon%=360
    
    DeltaPsi,DeltaEps,Eps0,Eps=nutation(J,UT,Delta_T)
    lambdaMoonApparent=lambdaMoon+DeltaPsi
    LMoon=lambdaMoonApparent
         
    #sumR
    cosine=[]
    z=0.0
    for i in range(0,60):
        z=cfCosine[i]*cosd(cfD[i]*D+cfM[i]*M+cfMdash[i]*Mdash+cfF[i]*F)
        if cfM[i]==1 or cfM[i]==-1:
            z*=E
        elif cfM[i]==2 or cfM[i]==-2:
            z=z*E*E            
        cosine.append(z)

    sumR=math.fsum(cosine)    
      
    deltaMoon=385000.56+sumR/1000    
    
    #Coefficients for sumB
    cfD=[0,0,0,2,2,2,2,0,2,0,2,2,2,2,2,2,2,0,4,0,0,0,1,0,0,0,1,0,4,4,0,4,2,2,2,2,0,2,2,2,2,4,2,2,0,2,1,1,0,2,1,2,0,4,4,1,4,1,4,2]
    cfM=[0,0,0,0,0,0,0,0,0,0,-1,0,0,1,-1,-1,-1,1,0,1,0,1,0,1,1,1,0,0,0,0,0,0,0,0,-1,0,0,0,0,1,1,0,-1,-2,0,1,1,1,1,1,0,-1,1,0,-1,0,0,0,-1,-2]
    cfMdash=[0,1,1,0,-1,-1,0,2,1,2,0,-2,1,0,-1,0,-1,-1,-1,0,0,-1,0,1,1,0,0,3,0,-1,1,-2,0,2,1,-2,3,2,-3,-1,0,0,1,0,1,1,0,0,-2,-1,1,-2,2,-2,-1,1,1,-1,0,0,]
    cfF=[1,1,-1,-1,1,-1,1,1,-1,-1,-1,-1,1,-1,1,1,-1,-1,-1,1,3,1,1,1,-1,-1,-1,1,-1,1,-3,1,-3,-1,-1,1,-1,1,-1,1,1,1,1,-1,3,-1,-1,1,-1,-1,1,-1,1,-1,-1,-1,-1,-1,-1,1]
        
    cfSine=[5128122,280602,277693,173237,55413,46271,32573,17198,9266,8822,8216,4324,4200,-3359,2463,2211,2065,-1870,1828,-1794,-1749,-1565,-1491,-1475,-1410,-1344,-1335,1107,1021,833,777,671,607,596,491,-451,439,422,421,-366,-351,331,315,302,-283,-229,223,223,-220,-220,-185,181,-177,176,166,-164,132,-119,115,107]

    #sumB
    sine=[]
    z=0.0
    for i in range(0,60):
        z=cfSine[i]*sind(cfD[i]*D+cfM[i]*M+cfMdash[i]*Mdash+cfF[i]*F)                   
        sine.append(z)

    sumB=math.fsum(sine)
    sumB=sumB-2235*sind(Ldash)+382*sind(A_3)+175*sind(A_1-F)+175*sind(A_1+F)+127*sind(Ldash-Mdash)-115*sind(Ldash+Mdash)
    
    betaMoon=sumB/1000000    
       
    RAMoon=atand2(sind(lambdaMoonApparent)*cosd(Eps)-tand(betaMoon)*sind(Eps),cosd(lambdaMoonApparent))
    RAMoon%=360
       
    DecMoon=asind(sind(betaMoon)*cosd(Eps)+cosd(betaMoon)*sind(Eps)*sind(lambdaMoonApparent))
    
    GMST=280.46061837+360.98564736629*(JD-2451545)+T*T*(0.000387933-T/38710000)    
       
    GAST=GMST+DeltaPsi*cosd(Eps)    
    
    GHAMoon=GAST-RAMoon
    GHAMoon%=360
    
    #Measured in degrees
    HPMoon=asind(6378.14/deltaMoon)
    SDMoon=asind(1738/deltaMoon)
    
    #Illumination
    psi=acosd(sind(Dec)*sind(DecMoon)+cosd(Dec)*cosd(DecMoon)*cosd(RA-RAMoon))
    deltaSun=1.496e8*R
    i=atand2(deltaSun*sind(psi),(deltaMoon-deltaSun*cosd(psi)))
    k=(1+cosd(i))/2
      
    return RAMoon,DecMoon,GHAMoon,HPMoon,SDMoon,k,LMoon
     
print()
print("Astronomical Almanac for Sun and Moon 0.92 (experimental!)            H. Umland 2019")
print("------------------------------------------------------------------------------------")
print("Update N (=TAI-UTC) in line 633 and DUT1 in line 634 if necessary (IERS Bulletin A).")
print("====================================================================================")

# Sun coordinates
print()
date=input("Enter date and UT (format: yyyymmdd.hhmmss or yyyymmdd.hhmmss.s): ")
if len(date)!=15 and len(date)!=17:
    print()
    print("Incorrect input format! Restart program.")
    exit()
year=int(date[0:4])
month=int(date[4:6])
day=int(date[6:8])
hour=int(date[9:11])
minute=int(date[11:13])
second=float(date[13:])
UT=hour+minute/60+second/3600

#Update N and DUT if necessary (see IERS Bulletin A)
N=37 #=TAI-UTC
DUT1=-0.2 #as of May 2, 2019

Delta_T=32.184+N-DUT1

J=date2jd(year,month,day)

wd=weekday(J)

mn=monthname(month)

GMST,GAST,RA,GHA,Dec,SD,HP,EoT,R,LSun=suncoordinates(J,UT,Delta_T)
RAMoon,DecMoon,GHAMoon,HPMoon,SDMoon,k,LMoon=mooncoordinates(J,UT,Delta_T)

DiffLSunLMoon=LSun-LMoon

if Dec<0:
    sigDec="S"
else:
    sigDec="N"
    
absDec=abs(Dec)
    
if DecMoon<0:
    sigDecMoon="S"
else:
    sigDecMoon="N"
    
absDecMoon=abs(DecMoon)

RAMoon1,DecMoon1,GHAMoon1,HPMoon1,SDMoon1,k1,LMoon1=mooncoordinates(J,UT-0.01,Delta_T)
RAMoon2,DecMoon2,GHAMoon2,HPMoon2,SDMoon2,k2,LMoon2=mooncoordinates(J,UT+0.01,Delta_T)
phase=""
if k2>k1:
    phase="(waxing)"
elif k2<k1:
    phase="(waning)"
    
#Date of next full Moon, bisection method
J0=J-1
J1=J+30
for i in range(0,200):
    Jmean=(J0+J1)/2
    GMST0,GAST0,RA0,GHA0,Dec0,SD0,HP0,EoT0,R0,LSun0=suncoordinates(J0,0,Delta_T)
    GMST1,GAST1,RA1,GHA1,Dec1,SD1,HP1,EoT1,R1,LSun1=suncoordinates(J1,0,Delta_T)
    GMSTmean,GASTmean,RAmean,GHAmean,Decmean,SDmean,HPmean,EoTmean,Rmean,LSunmean=suncoordinates(Jmean,0,Delta_T)
    RAMoon0,DecMoon0,GHAMoon0,HPMoon0,SDMoon0,kMoon0,LMoon0=mooncoordinates(J0,0,Delta_T)
    RAMoon1,DecMoon1,GHAMoon1,HPMoon1,SDMoon1,kMoon1,LMoon1=mooncoordinates(J1,0,Delta_T)
    RAMoonmean,DecMoonmean,GHAMoonmean,HPMoonmean,SDMoonmean,kMoonmean,LMoonmean=mooncoordinates(Jmean,0,Delta_T)
    
    Diffmean=LSunmean-LMoonmean
    Diffmean%=360
    sinDiff=sind(Diffmean)
    
    if sinDiff>0:
        J1=Jmean
    elif sinDiff<0:
        J0=Jmean
              
    if math.isclose(Diffmean,180,abs_tol=1e-7) or math.isclose(Diffmean,-180,abs_tol=1e-7):
        break

Jfull=Jmean
day_FullMoon,month_FullMoon,year_FullMoon=jd2date(Jfull)
UTfull=(Jfull-int(Jfull)+0.5)*24
Jfull,UTfull=normtime(Jfull,UTfull)
hfull,mfull=dh2hm(UTfull)

#Date of next new Moon, bisection method
J0=J-1
J1=J+30
for i in range(0,200):
    Jmean=(J0+J1)/2
    GMST0,GAST0,RA0,GHA0,Dec0,SD0,HP0,EoT0,R0,LSun0=suncoordinates(J0,0,Delta_T)
    GMST1,GAST1,RA1,GHA1,Dec1,SD1,HP1,EoT1,R1,LSun1=suncoordinates(J1,0,Delta_T)
    GMSTmean,GASTmean,RAmean,GHAmean,Decmean,SDmean,HPmean,EoTmean,Rmean,LSunmean=suncoordinates(Jmean,0,Delta_T)
    RAMoon0,DecMoon0,GHAMoon0,HPMoon0,SDMoon0,kMoon0,LMoon0=mooncoordinates(J0,0,Delta_T)
    RAMoon1,DecMoon1,GHAMoon1,HPMoon1,SDMoon1,kMoon1,LMoon1=mooncoordinates(J1,0,Delta_T)
    RAMoonmean,DecMoonmean,GHAMoonmean,HPMoonmean,SDMoonmean,kMoonmean,LMoonmean=mooncoordinates(Jmean,0,Delta_T)
    
    Diffmean=LSunmean-LMoonmean
    Diffmean%=360
    sinDiff=sind(Diffmean)
    
    if sinDiff<0:
        J1=Jmean
    elif sinDiff>0:
        J0=Jmean
   
    if math.isclose(Diffmean,0,abs_tol=1e-7) or math.isclose(Diffmean,360,abs_tol=1e-7):
        break
    
#Jnew=Jmean+UT/24
Jnew=Jmean
day_NewMoon,month_NewMoon,year_NewMoon=jd2date(Jnew)
UTnew=(Jnew-int(Jnew)+0.5)*24
Jnew,UTnew=normtime(Jnew,UTnew)
hnew,mnew=dh2hm(UTnew)

h,m,s=dd2ddmmss_s(UT)

print()
print()
print("Astronomical data for ",wd,", ",year," ",mn," ",day,",  ",h,":",m,":",s," UT",sep="")
print("------------------------------------------------------------------------------------")
print("Delta T:                          ",round(Delta_T,1)," s",sep="")
print()

h,m,s=dd2ddmmss_s(GMST/15)
print("Greenwich Mean Sidereal Time:     ",round(GMST,6),"°  =  ",h,":",m,":",s," GMST",sep="")

h,m,s=dd2ddmmss_s(GAST/15)
print()
print("Greenwich Apparent Sidereal Time: ",round(GAST,6),"°  =  ",h,":",m,":",s," GAST",sep="")

print("------------------------------------------------------------------------------------")
print("Sun data (apparent geocentric):")

RAh=RA/15
h,m,s=dd2ddmmss_s(RAh)
print()
print("Right ascension:               ",round(RA,6),"°  =  ",h," h ",m," m ",s," s",sep="")

h,m,s=dd2dms(GHA)
print()
print("Greenwich hour angle:          ",round(GHA,6),"°  =  ",h,"° ",m,"' ",s,"''",sep="")

h,m,s=dd2dms(absDec)
print()
print("Declination:                   ",round(Dec,6),"°  =  ",sigDec," ",h,"° ",m,"' ",s,"''",sep="")

print()
print("Semidiameter:                  ",round(SD,2),"'",sep="")

print()
print("Horizontal parallax:           ",round(HP,2),"'",sep="")

#EoT in minutes and seconds of time    
if EoT<0:
    sigEoT="-"
else:
    sigEoT="+"
absEoT=4*abs(EoT)
EoTmin=int(absEoT)
EoTsec=60*(absEoT-EoTmin)
print()
print("Equation of time:           ",sigEoT,EoTmin,"m",round(EoTsec,1),"s")
print("------------------------------------------------------------------------------------")

print("Moon data (apparent geocentric):")
RAMoonh=RAMoon/15
h,m,s=dd2dms(RAMoonh)
print()
print("Right ascension:               ",round(RAMoon,3),"°  =  ",h," h ",m," m ",s," s",sep="")
h,m,s=dd2dms(GHAMoon)
print()
print("Greenwich hour angle:          ",round(GHAMoon,3),"°  =  ",h,"° ",m,"' ",s,"''",sep="")
h,m,s=dd2dms(absDecMoon)
print()
print("Declination:                   ",round(DecMoon,3),"°  =  ",sigDecMoon," ",h,"° ",m,"' ",s,"''",sep="")
print()
print("Semidiameter:                  ",round(60*SDMoon,2),"'",sep="")
print()
print("Horizontal parallax:           ",round(60*HPMoon,2),"'",sep="")
print()
print("Illuminated fraction of disk:  ",round(100*k,1),"% ",phase,sep="")
print()
print("Full Moon:                     ",year_FullMoon,"-",month_FullMoon,"-",int(day_FullMoon),"   ",hfull,":",mfull," UT",sep="")
print()
print("New Moon:                      ",year_NewMoon,"-",month_NewMoon,"-",int(day_NewMoon),"   ",hnew,":",mnew," UT",sep="")
print("====================================================================================")

print()
print("Observer's position:")
print()
#print("Enter latitude in decimal degrees  (negative number for southern latitude):")
Lat=float(input("Latitude in decimal degrees  (negative number for southern latitude): "))
print()
#print("Enter longitude in decimal degrees (negative number for western longitude):")
Lon=float(input("Longitude in decimal degrees (negative number for western longitude): "))
print("====================================================================================")

#Altitude & azimuth
t=normt(GHA+Lon)

Hc=90-ahavd(havd(Lat-Dec)+cosd(Lat)*cosd(Dec)*havd(t))
AzN=atand2(sind(t),sind(Lat)*cosd(t)-cosd(Lat)*tand(Dec))+180

if Hc<0:
    sigHc="-"
else:
    sigHc="+"

absHc=abs(Hc)

tMoon=GHAMoon+Lon
if tMoon>180.0:
    tMoon-=360.0

HcMoon=90-ahavd(havd(Lat-DecMoon)+cosd(Lat)*cosd(DecMoon)*havd(tMoon))
AzNMoon=atand2(sind(tMoon),sind(Lat)*cosd(tMoon)-cosd(Lat)*tand(DecMoon))+180

if HcMoon<0:
    sigHcMoon="-"
else:
    sigHcMoon="+"

absHcMoon=abs(HcMoon)

u,v,w=dd2dms(absHc)
print()
print("Sun data:")
print()
print("Altitude of Sun (geocentric):      ",round(Hc,6),"°","  =  ",sigHc,u,"° ",v,"' ",w,"''",sep="")
print()
print("Azimuth of Sun (AzN):              ",round(AzN,6),"°",sep="")

#Meridian transit of Sun
UTtrans=12-(EoT+Lon)/15 #approximate value

t=0.1
while not math.isclose(t,0,abs_tol=1e-7):

    GMST,GAST,RA,GHA,Dec,SD,HP,EoT,R,LSun=suncoordinates(J,UTtrans,Delta_T)
      
    t=normt(GHA+Lon)
             
    UTtrans=12-(EoT+Lon)/15
          
Jtrans=J
    
Jtrans,UTtrans=normtime(Jtrans,UTtrans)
   
HcSunTrans=90-math.fabs(Dec-Lat)
       
day_trans,month_trans,year_trans=jd2date(Jtrans)

#Begin astronomical twilight
try:
    t=-acosd((sind(-18)-sind(Dec)*sind(Lat))/cosd(Dec)/cosd(Lat))
except:
    t=-180

try:
    UTbat=UTtrans+t/15 #approximate value
    message_bat=0
    H=-18.1
    while not math.isclose(H,-18,abs_tol=1e-7):
    
        GMST,GAST,RA,GHA,Dec,SD,HP,EoT,R,LSun=suncoordinates(J,UTbat,Delta_T)
    
        t=-acosd((sind(-18)-sind(Dec)*sind(Lat))/cosd(Dec)/cosd(Lat))
            
        H=asind(sind(Lat)*sind(Dec)+cosd(Lat)*cosd(Dec)*cosd(t))       
    
        UTbat=12-EoT/15-Lon/15+t/15
    
    Jbat=J
    Jbat,UTbat=normtime(Jbat,UTbat)
         
    day_bat,month_bat,year_bat=jd2date(Jbat)

except:
    message_bat=1

#Begin nautical twilight
try:
    t=-acosd((sind(-12)-sind(Dec)*sind(Lat))/cosd(Dec)/cosd(Lat))
except:
    t=-180    
    
try:
    UTbnt=UTtrans+t/15 #approximate value
    message_bnt=0
    H=-12.1
    while not math.isclose(H,-12,abs_tol=1e-7):
    
        GMST,GAST,RA,GHA,Dec,SD,HP,EoT,R,LSun=suncoordinates(J,UTbnt,Delta_T)
    
        t=-acosd((sind(-12)-sind(Dec)*sind(Lat))/cosd(Dec)/cosd(Lat))
           
        H=asind(sind(Lat)*sind(Dec)+cosd(Lat)*cosd(Dec)*cosd(t))        
       
        UTbnt=12-EoT/15-Lon/15+t/15
    
    Jbnt=J
    Jbnt,UTbnt=normtime(Jbnt,UTbnt)
     
    day_bnt,month_bnt,year_bnt=jd2date(Jbnt)
    
except:
    message_bnt=1    

#Begin civil twilight
try:
    t=-acosd((sind(-6)-sind(Dec)*sind(Lat))/cosd(Dec)/cosd(Lat))
except:
    t=-180
    
try:
    UTbct=UTtrans+t/15 #approximate value
    message_bct=0
    H=-6.1
    while not math.isclose(H,-6,abs_tol=1e-7):
    
        GMST,GAST,RA,GHA,Dec,SD,HP,EoT,R,LSun=suncoordinates(J,UTbct,Delta_T)
    
        t=-acosd((sind(-6)-sind(Dec)*sind(Lat))/cosd(Dec)/cosd(Lat))
       
        H=asind(sind(Lat)*sind(Dec)+cosd(Lat)*cosd(Dec)*cosd(t))        
       
        UTbct=12-EoT/15-Lon/15+t/15
    
    Jbct=J
    Jbct,UTbct=normtime(Jbct,UTbct)
     
    day_bct,month_bct,year_bct=jd2date(Jbct)
    
except:
    message_bct=1

#Sunrise
try:
    t=-acosd((sind(-0.8333)-sind(Dec)*sind(Lat))/cosd(Dec)/cosd(Lat))
except:
    t=-180       
    
try:
    UTrise=UTtrans+t/15 #approximate value
    message_rise=0
    H=-0.8
    while not math.isclose(H,-0.8333,abs_tol=1e-7):
    
        GMST,GAST,RA,GHA,Dec,SD,HP,EoT,R,LSun=suncoordinates(J,UTrise,Delta_T)
    
        t=-acosd((sind(-0.8333)-sind(Dec)*sind(Lat))/cosd(Dec)/cosd(Lat))
           
        H=90-ahavd(havd(Lat-Dec)+cosd(Lat)*cosd(Dec)*havd(t))    
    
        UTrise=12-EoT/15-Lon/15+t/15
    
    AzNrise=atand2(sind(t),sind(Lat)*cosd(t)-cosd(Lat)*tand(Dec))+180
    AzNrise=round(AzNrise,6)
    
    Jrise=J
    Jrise,UTrise=normtime(Jrise,UTrise) 
     
    day_rise,month_rise,year_rise=jd2date(Jrise)
    
except:
    message_rise=1

#Sunset
try:
    t=acosd((sind(-0.8333)-sind(Dec)*sind(Lat))/cosd(Dec)/cosd(Lat))
except:
    t=180       
    
try:
    UTset=UTtrans+t/15 #approximate value
    message_set=0
    H=-0.8
    while not math.isclose(H,-0.8333,abs_tol=1e-7):
    
        GMST,GAST,RA,GHA,Dec,SD,HP,EoT,R,LSun=suncoordinates(J,UTset,Delta_T)
    
        t=acosd((sind(-0.8333)-sind(Dec)*sind(Lat))/cosd(Dec)/cosd(Lat))
       
        H=90-ahavd(havd(Lat-Dec)+cosd(Lat)*cosd(Dec)*havd(t))        
        
        UTset=12-EoT/15-Lon/15+t/15
    
    AzNset=atand2(sind(t),sind(Lat)*cosd(t)-cosd(Lat)*tand(Dec))+180
    AzNset=round(AzNset,6)
    
    Jset=J
    Jset,UTset=normtime(Jset,UTset)
     
    day_set,month_set,year_set=jd2date(Jset)

except:
    message_set=1

#End civil twilight
try:
    t=acosd((sind(-6)-sind(Dec)*sind(Lat))/cosd(Dec)/cosd(Lat))
except:
    t=180 
    
try:
    message_ect=0
    UTect=UTtrans+t/15 #approximate value
    H=-6.1
    while not math.isclose(H,-6,abs_tol=1e-7):
    
        GMST,GAST,RA,GHA,Dec,SD,HP,EoT,R,LSun=suncoordinates(J,UTect,Delta_T)
    
        t=acosd((sind(-6)-sind(Dec)*sind(Lat))/cosd(Dec)/cosd(Lat))
       
        H=asind(sind(Lat)*sind(Dec)+cosd(Lat)*cosd(Dec)*cosd(t))        
       
        UTect=12-EoT/15-Lon/15+t/15
    
    Ject=J
    Ject,UTect=normtime(Ject,UTect)
     
    day_ect,month_ect,year_ect=jd2date(Ject)
    
except:
    message_ect=1

#End nautical twilight
try:
    t=acosd((sind(-12)-sind(Dec)*sind(Lat))/cosd(Dec)/cosd(Lat))
except:
    t=180     
    
try:
    message_ent=0
    UTent=UTtrans+t/15 #approximate value
    H=-12.1
    while not math.isclose(H,-12,abs_tol=1e-7):
    
        GMST,GAST,RA,GHA,Dec,SD,HP,EoT,R,LSun=suncoordinates(J,UTent,Delta_T)
    
        t=acosd((sind(-12)-sind(Dec)*sind(Lat))/cosd(Dec)/cosd(Lat))
       
        H=asind(sind(Lat)*sind(Dec)+cosd(Lat)*cosd(Dec)*cosd(t))        
       
        UTent=12-EoT/15-Lon/15+t/15
    
    Jent=J
    Jent,UTent=normtime(Jent,UTent)
     
    day_ent,month_ent,year_ent=jd2date(Jent)
    
except:
    message_ent=1

#End astronomical twilight
try:
    t=acosd((sind(-18)-sind(Dec)*sind(Lat))/cosd(Dec)/cosd(Lat))
except:
    t=180    
    
try:
    message_eat=0
    UTeat=UTtrans+t/15 #approximate value
    H=-18.1
    while not math.isclose(H,-18,abs_tol=1e-7):
    
        GMST,GAST,RA,GHA,Dec,SD,HP,EoT,R,LSun=suncoordinates(J,UTeat,Delta_T)
    
        t=acosd((sind(-18)-sind(Dec)*sind(Lat))/cosd(Dec)/cosd(Lat))
        
        H=asind(sind(Lat)*sind(Dec)+cosd(Lat)*cosd(Dec)*cosd(t))        
       
        UTeat=12-EoT/15-Lon/15+t/15
    
    Jeat=J
    Jeat,UTeat=normtime(Jeat,UTeat)
     
    day_eat,month_eat,year_eat=jd2date(Jeat)

except:
    message_eat=1

#Output rise, transit, set, twilight
if message_bat==0:
    u,v=dh2hm(UTbat)
    print("------------------------------------------------------------------------------------")
    print("Begin astronomical twilight:       ",year_bat,"-",month_bat,"-",int(day_bat),"   ",u,":",v," UT",sep="")
elif message_bat==1:
    print()
    print("Begin astronomical twilight:      ","N.A.")

if message_bnt==0:
    u,v=dh2hm(UTbnt)
    print()
    print("Begin nautical twilight:           ",year_bnt,"-",month_bnt,"-",int(day_bnt),"   ",u,":",v," UT",sep="")
elif message_bnt==1:
    print()
    print("Begin nautical twilight:          ","N.A.")

if message_bct==0:
    u,v=dh2hm(UTbct)
    print()
    print("Begin civil twilight:              ",year_bct,"-",month_bct,"-",int(day_bct),"   ",u,":",v," UT",sep="")
elif message_bct==1:
    print()
    print("Begin civil twilight:             ","N.A.")
    
if message_rise==0:
    u,v=dh2hm(UTrise)
    print()
    print("Sunrise:                           ",year_rise,"-",month_rise,"-",int(day_rise),"   ",u,":",v," UT   ","  AzN: ",round(AzNrise,2),"°",sep="")
elif message_rise==1:
    print()
    print("Sunrise:                          ","N.A.")    


u,v,w=dd2dms(UTtrans)

print("------------------------------------------------------------------------------------")
print("Local apparent noon:               ",year_trans,"-",month_trans,"-",int(day_trans),"   ",u,":",v,":",w," UT  Alt.: ",round(HcSunTrans,3),"°",sep="")
print("------------------------------------------------------------------------------------")

if message_set==0:
    u,v=dh2hm(UTset)
    print("Sunset:                            ",year_set,"-",month_set,"-",int(day_set),"   ",u,":",v," UT   ","  AzN: ",round(AzNset,2),"°",sep="")
elif message_set==1:
    print("Sunset:                           ","N.A.")

if message_ect==0:
    u,v=dh2hm(UTect)
    print()
    print("End civil twilight:                ",year_ect,"-",month_ect,"-",int(day_ect),"   ",u,":",v," UT",sep="")
elif message_ect==1:
    print()
    print("End civil twilight:               ","N.A.")
   
if message_ent==0:
    u,v=dh2hm(UTent)
    print()
    print("End nautical twilight:             ",year_ent,"-",month_ent,"-",int(day_ent),"   ",u,":",v," UT",sep="")
elif message_ent==1:
    print()
    print("End nautical twilight:            ","N.A.")
    
if message_eat==0:
    u,v=dh2hm(UTeat)
    print()
    print("End astronomical twilight:         ",year_eat,"-",month_eat,"-",int(day_eat),"   ",u,":",v," UT",sep="")
elif message_eat==1:    
    print()
    print("End astronomical twilight:        ","N.A.")
    
print("====================================================================================")
print()
print("Moon data:")
print()

u,v,w=dd2dms(absHcMoon)
print("Altitude of Moon (geocentric):     ",round(HcMoon,3),"°","  =  ",sigHcMoon,u,"° ",v,"' ",w,"''",sep="")
print()
print("Azimuth of Moon (AzN):             ",round(AzNMoon,3),"°",sep="")

#Moon transit same day
JMoonTrans=J
RAMoonTrans,DecMoonTrans,GHAMoonTrans,HPMoonTrans,SDMoonTrans,kMoonTrans,LMoonTrans=mooncoordinates(JMoonTrans,12,Delta_T)
tMoonTrans=normt(GHAMoonTrans+Lon)
UTMoonTrans=12-tMoonTrans/15
JMoonTrans,UTMoonTrans=normtime(JMoonTrans,UTMoonTrans)
     
for i in range(0,500):
    RAMoonTrans,DecMoonTrans,GHAMoonTrans,HPMoonTrans,SDMoonTrans,kMoonTrans,LMoonTrans=mooncoordinates(JMoonTrans,UTMoonTrans,Delta_T)
    tMoonTrans=normt(GHAMoonTrans+Lon)
    if math.isclose(tMoonTrans,0,abs_tol=1e-7):
        break
    UTMoonTrans=UTMoonTrans-tMoonTrans/15
    
JMoonTrans,UTMoonTrans=normtime(JMoonTrans,UTMoonTrans)
        
HcMoonTrans=90-math.fabs(DecMoonTrans-Lat)
day_Moon_Trans,month_Moon_Trans,year_Moon_Trans=jd2date(JMoonTrans)
htrans,mtrans,strans=dd2dms(UTMoonTrans)

#Moon transit preceding day
JMoonTransPrec=J-1
RAMoonTransPrec,DecMoonTransPrec,GHAMoonTransPrec,HPMoonTransPrec,SDMoonTransPrec,kMoonTransPrec,LMoonTransPrec=mooncoordinates(JMoonTransPrec,12,Delta_T)
tMoonTransPrec=normt(GHAMoonTransPrec+Lon)
UTMoonTransPrec=12-tMoonTransPrec/15
JMoonTransPrec,UTMoonTransPrec=normtime(JMoonTransPrec,UTMoonTransPrec)
     
for i in range(0,500):
    RAMoonTransPrec,DecMoonTransPrec,GHAMoonTransPrec,HPMoonTransPrec,SDMoonTransPrec,kMoonTransPrec,LMoonTRansPrec=mooncoordinates(JMoonTransPrec,UTMoonTransPrec,Delta_T)
    tMoonTransPrec=normt(GHAMoonTransPrec+Lon)
    if math.isclose(tMoonTransPrec,0,abs_tol=1e-7):
        break
    UTMoonTransPrec=UTMoonTransPrec-tMoonTransPrec/15
    JMoonTransPrec,UTMoonTransPrec=normtime(JMoonTransPrec,UTMoonTransPrec)
    if JMoonTransPrec==J:
       JMoonTransPrec-=1
        
HcMoonTransPrec=90-math.fabs(DecMoonTransPrec-Lat)
day_Moon_TransPrec,month_Moon_TransPrec,year_Moon_TransPrec=jd2date(JMoonTransPrec)
htransPrec,mtransPrec,stransPrec=dd2dms(UTMoonTransPrec)

#Moon transit following day
JMoonTransFoll=J+1
RAMoonTransFoll,DecMoonTransFoll,GHAMoonTransFoll,HPMoonTransFoll,SDMoonTransFoll,kMoonTransFoll,LMoonTransFoll=mooncoordinates(JMoonTransFoll,12,Delta_T)
tMoonTransFoll=normt(GHAMoonTransFoll+Lon)
UTMoonTransFoll=12-tMoonTransFoll/15
JMoonTransFoll,UTMoonTransFoll=normtime(JMoonTransFoll,UTMoonTransFoll)
     
for i in range(0,500):
    RAMoonTransFoll,DecMoonTransFoll,GHAMoonTransFoll,HPMoonTransFoll,SDMoonTransFoll,kMoonTransFoll,LMoonTransFoll=mooncoordinates(JMoonTransFoll,UTMoonTransFoll,Delta_T)
    tMoonTransFoll=normt(GHAMoonTransFoll+Lon)
    if math.isclose(tMoonTransFoll,0,abs_tol=1e-7):
        break
    UTMoonTransFoll=UTMoonTransFoll-tMoonTransFoll/15
    JMoonTransFoll,UTMoonTransFoll=normtime(JMoonTransFoll,UTMoonTransFoll)
    if JMoonTransFoll==J:
        JMoonTransFoll+=1   

HcMoonTransFoll=90-math.fabs(DecMoonTransFoll-Lat)
day_Moon_TransFoll,month_Moon_TransFoll,year_Moon_TransFoll=jd2date(JMoonTransFoll)
htransFoll,mtransFoll,stransFoll=dd2dms(UTMoonTransFoll)

#Moonset preceding day, bisection method
UTMoon0Prec=UTMoonTransPrec
UTMoon1Prec=UTMoonTransPrec+13
MoonsetPrec=1
for i in range(0,500):
    UTMoonmeanPrec=(UTMoon0Prec+UTMoon1Prec)/2
    RAMoon0Prec,DecMoon0Prec,GHAMoon0Prec,HPMoon0Prec,SDMoon0Prec,kMoon0Prec,LMoon0Prec=mooncoordinates(JMoonTransPrec,UTMoon0Prec,Delta_T)
    H0Prec=90-ahavd(havd(Lat-DecMoon0Prec)+cosd(Lat)*cosd(DecMoon0Prec)*havd(GHAMoon0Prec+Lon))
    RAMoon1Prec,DecMoon1Prec,GHAMoon1Prec,HPMoon1Prec,SDMoon1Prec,kMoon1Prec,LMoon1Prec=mooncoordinates(JMoonTransPrec,UTMoon1Prec,Delta_T)
    H1Prec=90-ahavd(havd(Lat-DecMoon1Prec)+cosd(Lat)*cosd(DecMoon1Prec)*havd(GHAMoon1Prec+Lon))
    RAMoonmeanPrec,DecMoonmeanPrec,GHAMoonmeanPrec,HPMoonmeanPrec,SDMoonmeanPrec,kMoonmeanPrec,LMoonmeanPrec=mooncoordinates(JMoonTransPrec,UTMoonmeanPrec,Delta_T)
    HmeanPrec=90-ahavd(havd(Lat-DecMoonmeanPrec)+cosd(Lat)*cosd(DecMoonmeanPrec)*havd(GHAMoonmeanPrec+Lon))
    derivm=moonslope(JMoonTransPrec,UTMoonmeanPrec,Delta_T)
    if HmeanPrec<(0.7275*HPMoonmeanPrec-0.5667) and derivm<0:
        UTMoon1Prec=UTMoonmeanPrec
    elif HmeanPrec>(0.7275*HPMoonmeanPrec-0.5667) and derivm<0:
        UTMoon0Prec=UTMoonmeanPrec
    elif derivm>=0:
        MoonsetPrec=0
        break
    if math.isclose(UTMoon0Prec,UTMoon1Prec,abs_tol=1e-7):
        break

UTMoonsetPrec=UTMoonmeanPrec
tMoonsetPrec=normt(GHAMoonmeanPrec+Lon)

JMoonsetPrec=JMoonTransPrec
JMoonsetPrec,UTMoonsetPrec=normtime(JMoonsetPrec,UTMoonsetPrec)
RAMoonsetPrec,DecMoonsetPrec,GHAMoonsetPrec,HPMoonsetPrec,SDMoonsetPrec,kMoonsetPrec,LMoonsetPrec=mooncoordinates(JMoonsetPrec,UTMoonsetPrec,Delta_T)
  
AzNMoonsetPrec=atand2(sind(tMoonsetPrec),sind(Lat)*cosd(tMoonsetPrec)-cosd(Lat)*tand(DecMoonsetPrec))+180
AzNMoonsetPrec=round(AzNMoonsetPrec,2)
   
day_MoonsetPrec,month_MoonsetPrec,year_MoonsetPrec=jd2date(JMoonsetPrec)    
hsetPrec,msetPrec=dh2hm(UTMoonsetPrec)

#Moonrise, bisection method
UTMoon0=UTMoonTrans-13
UTMoon1=UTMoonTrans
Moonrise=1
for i in range(0,500):
    UTMoonmean=(UTMoon0+UTMoon1)/2
    RAMoon0,DecMoon0,GHAMoon0,HPMoon0,SDMoon0,kMoon0,LMoon0=mooncoordinates(JMoonTrans,UTMoon0,Delta_T)
    H0=90-ahavd(havd(Lat-DecMoon0)+cosd(Lat)*cosd(DecMoon0)*havd(GHAMoon0+Lon))
    RAMoon1,DecMoon1,GHAMoon1,HPMoon1,SDMoon1,kMoon1,LMoon0=mooncoordinates(JMoonTrans,UTMoon1,Delta_T)
    H1=90-ahavd(havd(Lat-DecMoon1)+cosd(Lat)*cosd(DecMoon1)*havd(GHAMoon1+Lon))
    RAMoonmean,DecMoonmean,GHAMoonmean,HPMoonmean,SDMoonmean,kMoonmean,LMoonmean=mooncoordinates(JMoonTrans,UTMoonmean,Delta_T)
    Hmean=90-ahavd(havd(Lat-DecMoonmean)+cosd(Lat)*cosd(DecMoonmean)*havd(GHAMoonmean+Lon))
    derivm=moonslope(JMoonTrans,UTMoonmean,Delta_T)
    if Hmean>(0.7275*HPMoonmean-0.5667) and derivm>0:
        UTMoon1=UTMoonmean
    elif Hmean<(0.7275*HPMoonmean-0.5667) and derivm>0:
        UTMoon0=UTMoonmean
    elif derivm<=0:
        Moonrise=0
        break    
    if math.isclose(UTMoon0,UTMoon1,abs_tol=1e-7):
        break

UTMoonrise=UTMoonmean
tMoonrise=normt(GHAMoonmean+Lon)

JMoonrise=JMoonTrans
JMoonrise,UTMoonrise=normtime(JMoonrise,UTMoonrise)
RAMoonrise,DecMoonrise,GHAMoonrise,HPMoonrise,SDMoonrise,kMoonrise,LMoonrise=mooncoordinates(JMoonrise,UTMoonrise,Delta_T)
    
AzNMoonrise=atand2(sind(tMoonrise),sind(Lat)*cosd(tMoonrise)-cosd(Lat)*tand(DecMoonrise))+180
AzNMoonrise=round(AzNMoonrise,2)

day_Moonrise,month_Moonrise,year_Moonrise=jd2date(JMoonrise)    
hrise,mrise=dh2hm(UTMoonrise)

#Moonset, bisection method
UTMoon0=UTMoonTrans
UTMoon1=UTMoonTrans+13
Moonset=1
for i in range(0,500):
    UTMoonmean=(UTMoon0+UTMoon1)/2
    RAMoon0,DecMoon0,GHAMoon0,HPMoon0,SDMoon0,kMoon0,LMoon0=mooncoordinates(JMoonTrans,UTMoon0,Delta_T)
    H0=90-ahavd(havd(Lat-DecMoon0)+cosd(Lat)*cosd(DecMoon0)*havd(GHAMoon0+Lon))
    RAMoon1,DecMoon1,GHAMoon1,HPMoon1,SDMoon1,kMoon1,LMoon1=mooncoordinates(JMoonTrans,UTMoon1,Delta_T)
    H1=90-ahavd(havd(Lat-DecMoon1)+cosd(Lat)*cosd(DecMoon1)*havd(GHAMoon1+Lon))
    RAMoonmean,DecMoonmean,GHAMoonmean,HPMoonmean,SDMoonmean,kMoonmean,LMoonmean=mooncoordinates(JMoonTrans,UTMoonmean,Delta_T)
    Hmean=90-ahavd(havd(Lat-DecMoonmean)+cosd(Lat)*cosd(DecMoonmean)*havd(GHAMoonmean+Lon))
    derivm=moonslope(JMoonTrans,UTMoonmean,Delta_T)
    if Hmean<(0.7275*HPMoonmean-0.5667) and derivm<0:
        UTMoon1=UTMoonmean
    elif Hmean>(0.7275*HPMoonmean-0.5667) and derivm<0:
        UTMoon0=UTMoonmean
    elif derivm>=0:
        Moonset=0
        break
    if math.isclose(UTMoon0,UTMoon1,abs_tol=1e-7):
        break

UTMoonset=UTMoonmean
tMoonset=normt(GHAMoonmean+Lon)

JMoonset=JMoonTrans
JMoonset,UTMoonset=normtime(JMoonset,UTMoonset)
RAMoonset,DecMoonset,GHAMoonset,HPMoonset,SDMoonset,kMoonset,LMoonset=mooncoordinates(JMoonset,UTMoonset,Delta_T)
  
AzNMoonset=atand2(sind(tMoonset),sind(Lat)*cosd(tMoonset)-cosd(Lat)*tand(DecMoonset))+180
AzNMoonset=round(AzNMoonset,2)
   
day_Moonset,month_Moonset,year_Moonset=jd2date(JMoonset)    
hset,mset=dh2hm(UTMoonset)

#Moonrise following day, bisection method
UTMoon0Foll=UTMoonTransFoll-13
UTMoon1Foll=UTMoonTransFoll
MoonriseFoll=1
for i in range(0,500):
    UTMoonmeanFoll=(UTMoon0Foll+UTMoon1Foll)/2
    RAMoon0Foll,DecMoon0Foll,GHAMoon0Foll,HPMoon0Foll,SDMoon0Foll,kMoon0Foll,LMoon0Foll=mooncoordinates(JMoonTransFoll,UTMoon0Foll,Delta_T)
    H0Foll=90-ahavd(havd(Lat-DecMoon0Foll)+cosd(Lat)*cosd(DecMoon0Foll)*havd(GHAMoon0Foll+Lon))
    RAMoon1Foll,DecMoon1Foll,GHAMoon1Foll,HPMoon1Foll,SDMoon1Foll,kMoon1Foll,LMoon1Foll=mooncoordinates(JMoonTransFoll,UTMoon1Foll,Delta_T)
    H1Foll=90-ahavd(havd(Lat-DecMoon1Foll)+cosd(Lat)*cosd(DecMoon1Foll)*havd(GHAMoon1Foll+Lon))
    RAMoonmeanFoll,DecMoonmeanFoll,GHAMoonmeanFoll,HPMoonmeanFoll,SDMoonmeanFoll,kMoonmeanFoll,LMoonmeanFoll=mooncoordinates(JMoonTransFoll,UTMoonmeanFoll,Delta_T)
    HmeanFoll=90-ahavd(havd(Lat-DecMoonmeanFoll)+cosd(Lat)*cosd(DecMoonmeanFoll)*havd(GHAMoonmeanFoll+Lon))
    derivm=moonslope(JMoonTransFoll,UTMoonmeanFoll,Delta_T)
    if HmeanFoll>(0.7275*HPMoonmeanFoll-0.5667) and derivm>0:
        UTMoon1Foll=UTMoonmeanFoll
    elif HmeanFoll<(0.7275*HPMoonmeanFoll-0.5667) and derivm>0:
        UTMoon0Foll=UTMoonmeanFoll
    elif derivm<=0:
        MoonriseFoll=0
        break
    if math.isclose(UTMoon0Foll,UTMoon1Foll,abs_tol=1e-7):
        break

UTMoonriseFoll=UTMoonmeanFoll
tMoonriseFoll=normt(GHAMoonmeanFoll+Lon)

JMoonriseFoll=JMoonTransFoll
JMoonriseFoll,UTMoonriseFoll=normtime(JMoonriseFoll,UTMoonriseFoll)
RAMoonriseFoll,DecMoonriseFoll,GHAMoonriseFoll,HPMoonriseFoll,SDMoonriseFoll,kMoonriseFoll,LMoonriseFoll=mooncoordinates(JMoonriseFoll,UTMoonriseFoll,Delta_T)
  
AzNMoonriseFoll=atand2(sind(tMoonriseFoll),sind(Lat)*cosd(tMoonriseFoll)-cosd(Lat)*tand(DecMoonriseFoll))+180
AzNMoonriseFoll=round(AzNMoonriseFoll,2)
   
day_MoonriseFoll,month_MoonriseFoll,year_MoonriseFoll=jd2date(JMoonriseFoll)    
hriseFoll,mriseFoll=dh2hm(UTMoonriseFoll)

print("------------------------------------------------------------------------------------")
print("Preceding local meridian transit:  ",year_Moon_TransPrec,"-",month_Moon_TransPrec,"-",int(day_Moon_TransPrec),"   ",htransPrec,":",mtransPrec," UT   ","  Alt.: ",round(HcMoonTransPrec,3),"°",sep="")
print("------------------------------------------------------------------------------------")

if MoonsetPrec==0 or AzNMoonsetPrec==0 or AzNMoonsetPrec==180.0:
    print("Moonset:                           No Moonset on ",year_MoonsetPrec,"-",month_MoonsetPrec,"-",int(day_MoonsetPrec),sep="")
else:
    print("Moonset:                           ",year_MoonsetPrec,"-",month_MoonsetPrec,"-",int(day_MoonsetPrec),"   ",hsetPrec,":",msetPrec," UT   ","  AzN: ",AzNMoonsetPrec,"°",sep="")

print()
if Moonrise==0 or AzNMoonrise==0 or AzNMoonrise==180.0:
    print("Moonrise:                          No Moonrise on ",year_Moonrise,"-",month_Moonrise,"-",int(day_Moonrise),sep="")
else:
    print("Moonrise:                          ",year_Moonrise,"-",month_Moonrise,"-",int(day_Moonrise),"   ",hrise,":",mrise," UT     ","AzN: ",AzNMoonrise,"°",sep="")

print("------------------------------------------------------------------------------------")
print("Local meridian transit:            ",year_Moon_Trans,"-",month_Moon_Trans,"-",int(day_Moon_Trans),"   ",htrans,":",mtrans,":",strans," UT  Alt.: ",round(HcMoonTrans,3),"°",sep="")
print("------------------------------------------------------------------------------------")

if Moonset==0 or AzNMoonset==0 or AzNMoonset==180.0:
    print("Moonset:                           No Moonset on ",year_Moonset,"-",month_Moonset,"-",int(day_Moonset),sep="")
else:
    print("Moonset:                           ",year_Moonset,"-",month_Moonset,"-",int(day_Moonset),"   ",hset,":",mset," UT     ","AzN: ",AzNMoonset,"°",sep="")

print()
if MoonriseFoll==0 or AzNMoonriseFoll==0 or AzNMoonriseFoll==180.0:
    print("Moonrise:                          No Moonrise on ",year_MoonriseFoll,"-",month_MoonriseFoll,"-",int(day_MoonriseFoll),sep="")
else:
    print("Moonrise:                          ",year_MoonriseFoll,"-",month_MoonriseFoll,"-",int(day_MoonriseFoll),"   ",hriseFoll,":",mriseFoll," UT   ","  AzN: ",AzNMoonriseFoll,"°",sep="")

print("------------------------------------------------------------------------------------")
print("Following local meridian transit:  ",year_Moon_TransFoll,"-",month_Moon_TransFoll,"-",int(day_Moon_TransFoll),"   ",htransFoll,":",mtransFoll," UT   ","  Alt.: ",round(HcMoonTransFoll,3),"°",sep="")
print("------------------------------------------------------------------------------------")










